<?php

class PandaDocTokens extends SugarApi
{
    private static $relatedModules = array(
        'Opportunities' => array(
            'assigned_user_id' => 'Users',
            'account_id' => 'Accounts',
        )
    );

    /**
     * List of field names that can be used as token values for Object fields.
     *
     * @var array
     */
    private static $objectTokenFields = array(
        'name'
    );

    public function registerApiRest()
    {
        return array(
            'TokensApi' => array(
                'reqType' => 'GET',
                'path' => array('pandadoctokens'),
                'pathVars' => array(''),
                'method' => 'retrieveTokens',
                'shortHelp' => 'Custom end point for retrieving pandadoc tokens',
                'longHelp' => 'Custom end point for retrieving pandadoc tokens',
            ),
            'ConfigApi' => array(
                'reqType' => 'GET',
                'path' => array('pandadocconfig'),
                'pathVars' => array(''),
                'method' => 'retrieveConfig',
                'shortHelp' => 'Custom end point for retrieving pandadoc plugin configuration',
                'longHelp' => 'Custom end point for retrieving pandadoc plugin configuration',
            )
        );
    }

    public function retrieveTokens($api, $args)
    {
        $tokensList = array();
        if(!empty($args['module']) && !empty($args['id'])){
            $module = $args['module'];
            $modelId = $args['id'];

            $admin = new Administration();
            $pdSettings = $admin->retrieveSettings('pandadoc');
            $pdTokens = $pdSettings->settings['pandadoc_tokens'];
            /*
             * Fix for backward compatibility Sugar ver < 7.11.0
             * @see Administration::retrieveSettings()
            */
            $pdTokens = is_string($pdTokens) ? json_decode(html_entity_decode($pdTokens), true) : $pdTokens;

            $tokensList = $this->getModuleTokens($pdTokens, $module, $modelId);
            $model = BeanFactory::getBean($module, $modelId);

            if (!array_key_exists($module, static::$relatedModules)) {
                return $tokensList;
            }

            foreach(static::$relatedModules[$module] as $fieldName => $relatedModule) {
                $tokensList = array_merge(
                    $tokensList,
                    $this->getModuleTokens($pdTokens, $relatedModule, $model->$fieldName)
                );
            }
        }

        return $tokensList;
    }

    public function retrieveConfig($api, $args)
    {
        $admin = new Administration();
        $settings = $admin->retrieveSettings('pandadoc')->settings;
        $pandadocSettings = array();
        foreach($settings as $key => $value) {
            if (strpos($key, 'pandadoc') !== false) {
                $pandadocSettings[$key] = $value;
            }
        }
        return $pandadocSettings;
    }

    /**
     * Return all enabled to display tokens for module.
     *
     * @param array $allTokens List of all enabled tokens for all modules
     * @param string $module SugarCRM module name to take tokens from.
     * @param int $modelId Entity ID to take tokens from.
     * @return array List of tokens to display.
     */
    private function getModuleTokens($allTokens, $module, $modelId)
    {
        $model = BeanFactory::getBean($module, $modelId);
        if (!$modelId || !$model) {
            return array();
        }
        if (!array_key_exists($module, $allTokens)) {
           return array();
        }

        $currencyFields = $this->getBeanCurrencyFields($model);
        $tokensList = array();

        foreach($allTokens[$module] as $name=>$field) {
            if($field === "email"){
                $value = $model->emailAddress->getPrimaryAddress($model);
            }
            else if(is_numeric($model->$field) && in_array($field, $currencyFields)) {
                $value = number_format($model->$field, 2, '.', '');
            }
            else if(is_numeric($model->$field) && is_float((float)$model->$field)){
                $value = round($model->$field, 2);
            }
            else {
                $value = $model->$field;
            }

            $tokensList[$name] = $this->normalizeValue($value);
        }

        return $tokensList;
    }

    /**
     * Returns currency field names for passed bean.
     *
     * @param SugarBean $bean
     * @return string[]
     */
    private function getBeanCurrencyFields($bean)
    {
        $fields = $bean->getFieldDefinitions();
        $fieldNames = array();
        foreach ($fields as $name => $field) {
            if ($field['type'] === 'currency') {
                $fieldNames[] = $name;
            }
        }
        return $fieldNames;
    }

    /**
     * Executes cleaning and sanitizing on passed value.
     *
     * @param string|array $value
     * @return string Normalized value
     */
    private function normalizeValue($value)
    {
        if (is_array($value)) {
            $result = array();
            foreach ($value as $record) {
                $possibleFields = array_intersect(static::$objectTokenFields, array_keys(get_object_vars($record)));
                if ($possibleFields) {
                    $result[] = $record->{$possibleFields[0]};
                }
            }
            return explode(',', $result);
        }

        // Removing unnecessary characters.

        $value = trim($value, '^');
        $value = implode(',', explode('^,^', $value));

        return $value;
    }
}
